Attribute VB_Name = "OGLUtils"
Option Explicit

'Declaraes das funes da biblioteca que sero utilizadas.
Private Declare Function EnumDisplaySettings Lib "user32" Alias "EnumDisplaySettingsA" (ByVal lpszDeviceName As Long, ByVal iModeNum As Long, lpDevMode As Any) As Boolean
Private Declare Function ChangeDisplaySettings Lib "user32" Alias "ChangeDisplaySettingsA" (lpDevMode As Any, ByVal dwflags As Long) As Long
Private Declare Function CreateIC Lib "gdi32" Alias "CreateICA" (ByVal lpDriverName As String, ByVal lpDeviceName As String, ByVal lpOutput As String, ByVal lpInitData As Long) As Long

'Variveis de controle e inicializao
Public Key As Integer
Private hrc As Long

Private OldWidth As Long
Private OldHeight As Long
Private OldBits As Long
Private OldVertRefresh As Long

Private mPointerCount As Integer

'Variveis de comtrole dos movimentos
Global nAngle As Double
Global nCintura As Double
Global nBraco As Double
Global nDir As Integer
Global nDirBraco As Integer
Global nDirCintura As Integer
Global nBracoOn As Integer
Global nCinturaOn As Integer
Global nVelocidade As Double

Public Sub ReSizeGLScene(ByVal Width As GLsizei, ByVal Height As GLsizei)
'Inicializao do OpenGL
    ' Evitar a diviso por 0
    If Height = 0 Then
        Height = 1
    End If
    
    ' Inicializao da area de visualizao
    glViewport 0, 0, Width, Height
    'Inicializao da Matrix de projeo
    glMatrixMode mmProjection
    'glMatrixMode mmModelView
    glLoadIdentity
    'gluPerspective 45#, Width / Height, 0.1, 100#
     'glOrtho 5#, 100#, 5, 100, 0.1, 100#
     glOrtho -10#, 10#, -10, 10, -10, 100#
    'Inicializao da Matrix de visualizao
    glMatrixMode mmModelView
    glLoadIdentity
End Sub

Public Function InitGL() As Boolean
' Configurao do OpenGL
    ' suavizao de curvas
    glShadeModel smSmooth

    ' fundo preto
    glClearColor 0.9, 0.9, 0.8, 0#

    'Conf. do Z-Buffer
    glClearDepth 1#
    glEnable glcDepthTest
    glDepthFunc cfLEqual

    'Pespectiva melhorada
    glHint htPerspectiveCorrectionHint, hmNicest

    InitGL = True
End Function


Public Sub KillGLWindow()
' Destroi janela criada
    
    If hrc Then
        If wglMakeCurrent(0, 0) = 0 Then
            MsgBox "Release Of DC And RC Failed.", vbInformation, "SHUTDOWN ERROR"
        End If

        If wglDeleteContext(hrc) = 0 Then
            MsgBox "Release Rendering Context Failed.", vbInformation, "SHUTDOWN ERROR"
        End If
        hrc = 0
    End If

End Sub

Private Sub SaveCurrentScreen()
    ' Save the current screen resolution, bits, and Vertical refresh
    Dim ret As Long
    ret = CreateIC("DISPLAY", "", "", 0&)
    OldWidth = GetDeviceCaps(ret, HORZRES)
    OldHeight = GetDeviceCaps(ret, VERTRES)
    OldBits = GetDeviceCaps(ret, BITSPIXEL)
    OldVertRefresh = GetDeviceCaps(ret, VREFRESH)
    ret = DeleteDC(ret)
End Sub

Private Function FindDEVMODE(ByVal Width As Integer, ByVal Height As Integer, ByVal Bits As Integer, Optional ByVal VertRefresh As Long = -1) As DEVMODE
    ' locate a DEVMOVE that matches the passed parameters
    Dim ret As Boolean
    Dim i As Long
    Dim dm As DEVMODE
    i = 0
    Do  ' enumerate the display settings until we find the one we want
        ret = EnumDisplaySettings(0&, i, dm)
        If dm.dmPelsWidth = Width And _
            dm.dmPelsHeight = Height And _
            dm.dmBitsPerPel = Bits And _
            ((dm.dmDisplayFrequency = VertRefresh) Or (VertRefresh = -1)) Then Exit Do ' exit when we have a match
        i = i + 1
    Loop Until (ret = False)
    FindDEVMODE = dm
End Function

Private Sub ResetDisplayMode()
    Dim dm As DEVMODE             ' Device Mode
    
    dm = FindDEVMODE(OldWidth, OldHeight, OldBits, OldVertRefresh)
    dm.dmFields = DM_BITSPERPEL Or DM_PELSWIDTH Or DM_PELSHEIGHT
    If OldVertRefresh <> -1 Then
        dm.dmFields = dm.dmFields Or DM_DISPLAYFREQUENCY
    End If
    ' Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
    If (ChangeDisplaySettings(dm, CDS_FULLSCREEN) <> DISP_CHANGE_SUCCESSFUL) Then
    
        ' If The Mode Fails, Offer Two Options.  Quit Or Run In A Window.
        MsgBox "The Requested Mode Is Not Supported By Your Video Card", , "NeHe GL"
    End If

End Sub

Private Sub SetDisplayMode(ByVal Width As Integer, ByVal Height As Integer, ByVal Bits As Integer, ByRef fullscreen As Boolean, Optional VertRefresh As Long = -1)
    Dim dmScreenSettings As DEVMODE             ' Device Mode
    Dim p As Long
    SaveCurrentScreen                           ' save the current screen attributes so we can go back later
    
    dmScreenSettings = FindDEVMODE(Width, Height, Bits, VertRefresh)
    dmScreenSettings.dmBitsPerPel = Bits
    dmScreenSettings.dmPelsWidth = Width
    dmScreenSettings.dmPelsHeight = Height
    dmScreenSettings.dmFields = DM_BITSPERPEL Or DM_PELSWIDTH Or DM_PELSHEIGHT
    If VertRefresh <> -1 Then
        dmScreenSettings.dmDisplayFrequency = VertRefresh
        dmScreenSettings.dmFields = dmScreenSettings.dmFields Or DM_DISPLAYFREQUENCY
    End If
    ' Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
    If (ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN) <> DISP_CHANGE_SUCCESSFUL) Then
    
        ' If The Mode Fails, Offer Two Options.  Quit Or Run In A Window.
        If (MsgBox("The Requested Mode Is Not Supported By" & vbCr & "Your Video Card. Use Windowed Mode Instead?", vbYesNo + vbExclamation, "NeHe GL") = vbYes) Then
            fullscreen = False                  ' Select Windowed Mode (Fullscreen=FALSE)
        Else
            ' Pop Up A Message Box Letting User Know The Program Is Closing.
            MsgBox "Program Will Now Close.", vbCritical, "ERROR"
            End                   ' Exit And Return FALSE
        End If
    End If
End Sub

Public Function CreateGLWindow(frm As Form, Width As Integer, Height As Integer, Bits As Integer, fullscreenflag As Boolean) As Boolean
    Dim PixelFormat As GLuint                       ' Holds The Results After Searching For A Match
    Dim pfd As PIXELFORMATDESCRIPTOR                ' pfd Tells Windows How We Want Things To Be

    pfd.cColorBits = Bits
    pfd.cDepthBits = 16
    pfd.dwflags = PFD_DRAW_TO_WINDOW Or PFD_SUPPORT_OPENGL Or PFD_DOUBLEBUFFER
    pfd.iLayerType = PFD_MAIN_PLANE
    pfd.iPixelType = PFD_TYPE_RGBA
    pfd.nSize = Len(pfd)
    pfd.nVersion = 1
    
    PixelFormat = ChoosePixelFormat(frm.hdc, pfd)
    If PixelFormat = 0 Then                     ' Did Windows Find A Matching Pixel Format?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Find A Suitable PixelFormat.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If

    If SetPixelFormat(frm.hdc, PixelFormat, pfd) = 0 Then ' Are We Able To Set The Pixel Format?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Set The PixelFormat.", vbExclamation, "ERROR"
        CreateGLWindow = False                           ' Return FALSE
    End If
    
    hrc = wglCreateContext(frm.hdc)
    If (hrc = 0) Then                           ' Are We Able To Get A Rendering Context?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Create A GL Rendering Context.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If

    If wglMakeCurrent(frm.hdc, hrc) = 0 Then    ' Try To Activate The Rendering Context
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Activate The GL Rendering Context.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If
    frm.Show                                    ' Show The Window
    SetForegroundWindow frm.hWnd                ' Slightly Higher Priority
    frm.SetFocus                                ' Sets Keyboard Focus To The Window
    ReSizeGLScene frm.ScaleWidth, frm.ScaleHeight ' Set Up Our Perspective GL Screen

    If Not InitGL() Then                        ' Initialize Our Newly Created GL Window
        KillGLWindow                            ' Reset The Display
        MsgBox "Initialization Failed.", vbExclamation, "ERROR"
        CreateGLWindow = False                   ' Return FALSE
    End If
    'Foco de Luz
    glEnable GL_LIGHTING
    glEnable glcLight0
    CreateGLWindow = True

End Function
Private Sub Braco(ByRef nVal As Double, _
                  ByRef nList As Integer, _
                  ByRef nSide As Integer)
Dim obj&
obj = gluNewQuadric
    
    glNewList nList, lstCompile
        'Rot do brao
        glRotatef nVal * nSide, 0#, 0#, 1#
        'Brao para baixo
        glRotatef 90, 1#, 0#, 0#
        'Brao Sup.
        gluCylinder obj, 0.3, 0.3, 3, 10, 10
        'Ombro
        gluSphere obj, 0.5, 16, 16
        'Cotovelo
        glTranslatef 0#, 0#, 3
        gluSphere obj, 0.4, 16, 16
        'Rot. do brao inf
        glRotatef -nVal, 1#, 0#, 0#
        'Brao Inf
        gluCylinder obj, 0.3, 0.3, 2, 10, 10
        glTranslatef 0#, 0#, 2
        gluSphere obj, 0.3, 16, 16
    glEndList

End Sub
Private Sub Tronco(ByVal nList As Integer)
Dim obj&
obj = gluNewQuadric
    glNewList nList, lstCompile
        'Barriga
        glRotatef 90, 1#, 0#, 0#
        gluCylinder obj, 2.8, 2, 2, 10, 10
        'Peito
        glTranslatef 0#, 0#, -1#
        gluCylinder obj, 2.8, 2.8, 1, 10, 10
        glTranslatef 0#, 0#, -1#
        gluCylinder obj, 0, 2.8, 1, 10, 10
        'Cintura
        glTranslatef 0#, 0#, 4#
        gluSphere obj, 2, 20, 20
        'Cabea
        glTranslatef 0#, 0#, -4.5
        gluSphere obj, 1, 20, 20
        'Chapeu
        glTranslatef 0#, 0#, -1.5
        gluCylinder obj, 0, 1.5, 1, 30, 30
        'Olhos
        glTranslatef 0.5, 0.8, 1.3
        gluSphere obj, 0.2, 10, 10
        glTranslatef -1, 0#, 0#
        gluSphere obj, 0.2, 10, 10
        'Nariz
        glTranslatef 0.5, 0#, 0#
        gluCylinder obj, 0.2, 0.2, 0.5, 10, 10
        
    glEndList
End Sub
Private Sub Tudo(ByVal nList As Integer)
'Dim obj&
'obj = gluNewQuadric
    glNewList nList, lstCompile
        glCallList 3
        glCallList 3
        glCallList 3
        glCallList 3
        
    glEndList
End Sub
Private Sub Perna(ByVal nList As Integer, ByVal nSide As Integer)
Dim obj&
obj = gluNewQuadric
    
    glNewList nList, lstCompile
        'Perna
        glRotatef 90, 1#, 0#, 0#
        gluCylinder obj, 0.6, 0.6, 6, 10, 10
        'Lig. com a Cintura
        gluSphere obj, 0.8, 16, 16
        'Joelho
        glTranslatef 0#, 0#, 3
        gluSphere obj, 0.7, 16, 16
        'P
        glTranslatef 0#, 0#, 3
        gluCylinder obj, 0.9, 0.9, 0.3, 10, 10
    glEndList

End Sub
Public Function DrawGLScene(ByVal hdc As Long) As Boolean
Dim nRot As Double
Dim nDepth As Double
    'Limpa a tela
    glColor3f 1#, 0#, 0#
    glClear clrColorBufferBit Or clrDepthBufferBit
    'limpa matris
    glLoadIdentity
    
    'Distncia do Modelo
    nDepth = -25#
    'Inicializa as partes do corpo
    'Tronco
    Tronco 3
    'Brao esquerdo
    Braco nBraco, 1, 1
    'Brao direito
    Braco nBraco, 2, -1
    'Pernas
    Perna 4, 1
    
    'Monto a matris de rotao
    glLoadIdentity
    nRot = nCintura
    glTranslatef 0#, 0#, nDepth
    glRotatef nRot, 1#, 0#, 0#
'glRotatef 90, 0#, 0#, 1#
    'glPushMatrix & glPopMatrix : Uso para garantir que a situao atual
    '                             da matris de projeo no ser mudada
    '                             ao desenhar um objeto.
    glPushMatrix
      'Desenho o Tronco
      glTranslatef 0#, 3#, 0#
      glCallList 3
    glPopMatrix
    
    glPushMatrix
      'desenho Brao esquerdo
      glTranslatef 3#, 3#, 0#
      glCallList 1
    glPopMatrix
    
    glPushMatrix
      'desenho Brao direito
      glTranslatef -3#, 3#, 0#
      glCallList 2
    glPopMatrix

    'As pernas so fixas, logo limpo a matris de rotao.
    glLoadIdentity
'glRotatef 90, 0#, 0#, 1#
    'Desenho Perna esquerda
    glTranslatef 1#, -1#, nDepth#
    glCallList 4
    
    glLoadIdentity
    'Desenho Perna direita
 'glRotatef 90, 0#, 0#, 1#
    glTranslatef -1#, -1#, nDepth#
    glCallList 4
    
    If nAngle > 90 Or nAngle < 0 Then
        nDir = -nDir
    End If
    If nBraco > 90 Or nBraco < 0 Then
        nDirBraco = -nDirBraco
    End If
    If nCintura > 90 Or nCintura < 0 Then
        nDirCintura = -nDirCintura
    End If
    nAngle = nAngle + nVelocidade * nDir
    nCintura = nCintura + (nVelocidade * nDirCintura) * nCinturaOn
    nBraco = nBraco + (nVelocidade * nDirBraco) * nBracoOn
   
           

   
   DrawGLScene = True
End Function

Sub Main()
    Dim sMsg As String
    Dim Done As Boolean
    Dim frm As Form
    Done = False
    
      
    nBracoOn = 0
    nCinturaOn = 0
    nDir = 1
    nDirBraco = 1
    nDirCintura = 1
    nVelocidade = 1
    ' Cria a Janela
    Set frm = New Form1
    If Not CreateGLWindow(frm, 640, 480, 16, False) Then
        Done = True                             ' Quit If Window Was Not Created
    End If
    sMsg = "Funes:" & Chr(13) & _
            Chr(9) & " '1'  : Liga / Desliga Animao dos braos." & Chr(13) & _
            Chr(9) & " '2'  : Liga / Desliga Animao da cintura." & Chr(13) & _
            Chr(9) & "'+/-' : Aumenta / Diminui a velocidade de animao." & Chr(13) & _
            Chr(9) & "'W/w' : Liga Wire-frame." & Chr(13) & _
            Chr(9) & "'Q/q' : Desliga Wire-frame." & Chr(13) & _
            Chr(9) & "'S/s' : Liga suavizao de superfcie." & Chr(13) & _
            Chr(9) & "'A/a' : Desliga suavizao de superfcie." & Chr(13)
   ' MsgBox sMsg, vbOKOnly

    Do While Not Done
        ' Desenha  a cena at que se tecle ESC
        If (Not DrawGLScene(frm.hdc) Or Key = vbKeyEscape) Then
            Unload frm
        Else
            SwapBuffers (frm.hdc)                ' Double Buffering
            DoEvents
        End If

        'Verifico a ao de deve ser feita
        Select Case Key
          Case vbKey1
            'liga/desliga animao dos braos
            nBracoOn = IIf(nBracoOn = 0, 1, 0)
            Key = 0
          Case vbKey2
            'liga/desliga animao da cintura
            nCinturaOn = IIf(nCinturaOn = 0, 1, 0)
            Key = 0
          Case vbKeyS, (vbKeyS + 32)
            'liga suavizao de superfcie
            glShadeModel smSmooth
            Key = 0
          Case vbKeyA, (vbKeyA + 32)
            'desliga suavizao de superfcie
            glShadeModel smFlat
            Key = 0
          Case vbKeyW, (vbKeyW + 32)
            'liga wire frame
            glPolygonMode GL_FRONT_AND_BACK, GL_LINE
            Key = 0
          Case vbKeyQ, (vbKeyQ + 32)
            'desliga wire frame
            glPolygonMode GL_FRONT_AND_BACK, GL_FILL
            Key = 0
          Case 43
            'aumenta velocidade
            nVelocidade = nVelocidade * 2
            Key = 0
          Case 45
            'diminui velocidade
            nVelocidade = nVelocidade / 2
            Key = 0
        End Select
        Done = frm.Visible = False
    Loop
    'Finalizao
    Set frm = Nothing
    End
End Sub




